@@ -67,6 +67,10 @@
 
 
 
+// ChucK version string -- retrieve using ChucK::version()
+// 1.5.0.0 (ge) | moved here for at-a-glance visibility (e.g., for chugins)
+#define CHUCK_VERSION_STRING                "1.5.0.0 (chai)"
+
 // ChucK param names -- used in setParam(...) and getParam*(...)
 #define CHUCK_PARAM_SAMPLE_RATE             "SAMPLE_RATE"
 #define CHUCK_PARAM_INPUT_CHANNELS          "INPUT_CHANNELS"
@@ -103,10 +107,10 @@ public:
 public:
     // set parameter by name
     // -- all params should have reasonable defaults
-    bool setParam( const std::string & name, t_CKINT value );
-    bool setParamFloat( const std::string & name, t_CKFLOAT value );
-    bool setParam( const std::string & name, const std::string & value );
-    bool setParam( const std::string & name, const std::list< std::string > & value );
+    t_CKBOOL setParam( const std::string & name, t_CKINT value );
+    t_CKBOOL setParamFloat( const std::string & name, t_CKFLOAT value );
+    t_CKBOOL setParam( const std::string & name, const std::string & value );
+    t_CKBOOL setParam( const std::string & name, const std::list< std::string > & value );
     // get params
     t_CKINT getParamInt( const std::string & key );
     t_CKFLOAT getParamFloat( const std::string & key );
@@ -115,15 +119,15 @@ public:
 
 public:
     // compile a file (can be called anytime)
-    bool compileFile( const std::string & path, const std::string & argsTogether, t_CKINT count = 1 );
+    t_CKBOOL compileFile( const std::string & path, const std::string & argsTogether, t_CKINT count = 1 );
     // compile code directly
-    bool compileCode( const std::string & code, const std::string & argsTogether, t_CKINT count = 1 );
+    t_CKBOOL compileCode( const std::string & code, const std::string & argsTogether, t_CKINT count = 1 );
 
 public:
     // initialize ChucK (using params)
-    bool init();
+    t_CKBOOL init();
     // explicit start (optionally -- done as neede from run())
-    bool start();
+    t_CKBOOL start();
 
 public:
     // run engine (call from callback)
@@ -131,13 +135,13 @@ public:
 
 public:
     // is initialized
-    bool isInit() { return m_init; }
+    t_CKBOOL isInit() { return m_init; }
 
 public:
     // additional native chuck bindings/types (use with extra caution)
-    bool bind( f_ck_query queryFunc, const std::string & name );
+    t_CKBOOL bind( f_ck_query queryFunc, const std::string & name );
     // set a function pointer to call from the main thread loop
-    bool setMainThreadHook( Chuck_DL_MainThreadHook * hook );
+    t_CKBOOL setMainThreadHook( Chuck_DL_MainThreadHook * hook );
     // get pointer to current function to be called from main loop
     Chuck_DL_MainThreadHook * getMainThreadHook();
 
@@ -151,7 +155,7 @@ public:
     // get compiler (dangerous)
     Chuck_Compiler * compiler() { return m_carrier->compiler; }
     // is the VM running
-    bool vm_running() { return m_carrier->vm && m_carrier->vm->running(); }
+    t_CKBOOL vm_running() { return m_carrier->vm && m_carrier->vm->running(); }
 
 public:
     // global callback functions: replace printing to command line with a callback function
@@ -170,17 +174,15 @@ public:
 
 protected:
     // shutdown
-    bool shutdown();
+    t_CKBOOL shutdown();
 
 public: // static functions
     // chuck version
     static const char * version();
-    #ifndef __DISABLE_OTF_SERVER__
     // chuck int size (in bits)
-    // (this depends on machine, which depends on OTF, so
-    //  disable it if we don't have OTF)
+    // (this depends on machine, which depends on OTF, so disable it if we don't have OTF)
+    // 1.5.0.0 (ge) | reinstated for non-OTF, along with parts of machine
     static t_CKUINT intSize();
-    #endif
     // number of ChucK's
     static t_CKUINT numVMs() { return o_numVMs; };
     // --poop compatibilty
@@ -202,13 +204,13 @@ protected:
     // initialize default params
     void initDefaultParams();
     // init VM
-    bool initVM();
+    t_CKBOOL initVM();
     // init compiler
-    bool initCompiler();
+    t_CKBOOL initCompiler();
     // init chugin system
-    bool initChugins();
+    t_CKBOOL initChugins();
     // init OTF programming system
-    bool initOTF();
+    t_CKBOOL initOTF();
 
 protected:
     // core elements: compiler, VM, etc.
