@@ -84,6 +84,22 @@ typedef enum {
 
 
 
+//-----------------------------------------------------------------------------
+// name: enum te_Origin | 1.5.0.0 (ge) added
+// desc: where something (e.g., a Type) originates
+//-----------------------------------------------------------------------------
+typedef enum {
+    te_originUnknown = 0,
+    te_originBuiltin, // in core
+    te_originChugin, // in imported chugin
+    te_originImport, // library CK code
+    te_originUserDefined, // in user chuck code
+    te_originGenerated // generated (e.g., array types like int[][][][])
+} te_Origin;
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: struct Chuck_Scope
 // desc: scoping structure
@@ -182,7 +198,10 @@ public:
         else if( climb > 0 )
         {
             for( t_CKUINT i = scope.size(); i > 0; i-- )
-            { if( ( val = (*scope[i-1])[xid] ) ) break; }
+            {
+                val = (*scope[i - 1])[xid];
+                if( val ) break;
+            }
 
             // look in commit buffer
             if( !val && (commit_map.find(xid) != commit_map.end()) )
@@ -199,8 +218,15 @@ public:
         return (T)val;
     }
 
+    // test if a name has been "mangled", e.g., "toString@0@Object"
+    static t_CKBOOL is_mangled( const std::string & name )
+    {
+        // check for @ in the name, which would not be possible for names in language
+        return ( name.find("@") != std::string::npos );
+    }
+
     // get list of top level
-    void get_toplevel( std::vector<Chuck_VM_Object *> & out )
+    void get_toplevel( std::vector<Chuck_VM_Object *> & out, t_CKBOOL includeMangled = TRUE )
     {
         assert( scope.size() != 0 );
         std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
@@ -213,13 +239,16 @@ public:
         // go through map
         for( iter = m->begin(); iter != m->end(); iter++ )
         {
+            // check mangled name
+            if( !includeMangled && is_mangled( std::string(S_name((*iter).first))) )
+                continue;
             // add
             out.push_back( (*iter).second );
         }
     }
 
     // get list of top level
-    void get_level( int level, std::vector<Chuck_VM_Object *> & out )
+    void get_level( int level, std::vector<Chuck_VM_Object *> & out, t_CKBOOL includeMangled = TRUE )
     {
         assert( scope.size() >= level );
         std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
@@ -232,12 +261,15 @@ public:
         // go through map
         for( iter = m->begin(); iter != m->end(); iter++ )
         {
+            // check mangled name
+            if( !includeMangled && is_mangled( std::string(S_name((*iter).first))) )
+                continue;
+
             // add
             out.push_back( (*iter).second );
         }
     }
 
-
 protected:
     std::vector<std::map<S_Symbol, Chuck_VM_Object *> *> scope;
     std::map<S_Symbol, Chuck_VM_Object *> commit_map;
@@ -324,7 +356,7 @@ struct Chuck_Namespace : public Chuck_VM_Object
     // get top level values
     void get_values( std::vector<Chuck_Value *> & out );
     // get top level functions
-    void get_funcs( std::vector<Chuck_Func *> & out );
+    void get_funcs( std::vector<Chuck_Func *> & out, t_CKBOOL includeManged = TRUE );
 };
 
 
@@ -398,10 +430,15 @@ public:
     // constructor
     Chuck_Env();
 
+public:
+    // initialize
+    t_CKBOOL init();
+
 // REFACTOR-2017: carrier and accessors
 public:
     void set_carrier( Chuck_Carrier * carrier ) { m_carrier = carrier; }
-    Chuck_VM * vm() { return m_carrier->vm; }
+    Chuck_VM * vm() { return m_carrier ? m_carrier->vm : NULL; }
+    Chuck_Compiler * compiler() { return m_carrier ? m_carrier->compiler : NULL; }
 
 protected:
     Chuck_Carrier * m_carrier;
@@ -533,12 +570,10 @@ public:
     Chuck_Type * t_uanablob;
     Chuck_Type * t_shred;
     Chuck_Type * t_io;
-    #ifndef __DISABLE_FILEIO__
     Chuck_Type * t_fileio;
-    #endif
     Chuck_Type * t_chout;
     Chuck_Type * t_cherr;
-    Chuck_Type * t_thread;
+    // Chuck_Type * t_thread;
     Chuck_Type * t_class;
     Chuck_Type * t_dac;
     Chuck_Type * t_adc;
@@ -583,8 +618,9 @@ struct Chuck_UGen_Info : public Chuck_VM_Object
 //-----------------------------------------------------------------------------
 // name: struct Chuck_Type
 // desc: class containing information about a type
-//-----------------------------------------------------------------------------_id
-struct Chuck_Type : public Chuck_VM_Object
+// note: 1.5.0.0 (ge) Chuck_VM_Object -> Chuck_Object
+//-----------------------------------------------------------------------------
+struct Chuck_Type : public Chuck_Object
 {
     // type id
     te_Type xid;
@@ -620,6 +656,8 @@ struct Chuck_Type : public Chuck_VM_Object
     t_CKBOOL has_destructor;
     // custom allocator
     f_alloc allocator;
+    // origin hint
+    te_Origin originHint;
 
     // documentation
     std::string doc;
@@ -836,10 +874,14 @@ t_CKBOOL isa( Chuck_Type * lhs, Chuck_Type * rhs );
 t_CKBOOL isprim( Chuck_Env * env, Chuck_Type * type );
 t_CKBOOL isobj( Chuck_Env * env, Chuck_Type * type );
 t_CKBOOL isfunc( Chuck_Env * env, Chuck_Type * type );
+t_CKBOOL isvoid( Chuck_Env * env, Chuck_Type * type );
 t_CKBOOL iskindofint( Chuck_Env * env, Chuck_Type * type ); // added 1.3.1.0: this includes int + pointers
 t_CKUINT getkindof( Chuck_Env * env, Chuck_Type * type ); // added 1.3.1.0: to get the kindof a type
 
+
+//-----------------------------------------------------------------------------
 // import
+//-----------------------------------------------------------------------------
 Chuck_Type * type_engine_import_class_begin( Chuck_Env * env, Chuck_Type * type,
                                              Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor = NULL,
                                              const char * doc = NULL );
@@ -884,7 +926,10 @@ t_CKBOOL type_engine_import_class_end( Chuck_Env * env );
 t_CKBOOL type_engine_register_deprecate( Chuck_Env * env,
                                          const std::string & former, const std::string & latter );
 
-// helpers
+
+//-----------------------------------------------------------------------------
+// helper functions
+//-----------------------------------------------------------------------------
 t_CKBOOL type_engine_check_reserved( Chuck_Env * env, const std::string & xid, int pos );
 t_CKBOOL type_engine_check_reserved( Chuck_Env * env, S_Symbol xid, int pos );
 // 1.4.1.0 (ge): abilty to toggle reserved words for special cases, such as Math.pi co-existing with pi (use with care!)
@@ -893,17 +938,23 @@ t_CKBOOL type_engine_check_primitive( Chuck_Env * env, Chuck_Type * type );
 t_CKBOOL type_engine_check_const( Chuck_Env * env, a_Exp e, int pos ); // TODO
 t_CKBOOL type_engine_compat_func( a_Func_Def lhs, a_Func_Def rhs, int pos, std::string & err, t_CKBOOL print = TRUE );
 t_CKBOOL type_engine_get_deprecate( Chuck_Env * env, const std::string & from, std::string & to );
+t_CKBOOL type_engine_is_base_static( Chuck_Env * env, Chuck_Type * baseType ); // 1.5.0.0 (ge) added
 Chuck_Type  * type_engine_find_common_anc( Chuck_Type * lhs, Chuck_Type * rhs );
 Chuck_Type  * type_engine_find_type( Chuck_Env * env, a_Id_List path );
+Chuck_Type  * type_engine_find_type( Chuck_Env * env, const std::string & name ); // 1.5.0.0 (ge) added
 Chuck_Value * type_engine_find_value( Chuck_Type * type, const std::string & xid );
 Chuck_Value * type_engine_find_value( Chuck_Type * type, S_Symbol xid );
 Chuck_Value * type_engine_find_value( Chuck_Env * env, const std::string & xid, t_CKBOOL climb, int linepos = 0 );
 Chuck_Namespace * type_engine_find_nspc( Chuck_Env * env, a_Id_List path );
-/*******************************************************************************
- * spencer: added this into function to provide the same logic path
- * for type_engine_check_exp_decl() and ck_add_mvar() when they determine
- * offsets for mvars -- added 1.3.0.0
- ******************************************************************************/
+// convert a vector of type names to a vector of Types | 1.5.0.0 (ge) added
+void type_engine_names2types( Chuck_Env * env, const std::vector<std::string> & typeNames, std::vector<Chuck_Type *> & types );
+
+
+//-----------------------------------------------------------------------------
+// spencer: added this into function to provide the same logic path
+// for type_engine_check_exp_decl() and ck_add_mvar() when they determine
+// offsets for mvars -- added 1.3.0.0
+//-----------------------------------------------------------------------------
 t_CKUINT type_engine_next_offset( t_CKUINT current_offset, Chuck_Type * type );
 // array verify
 t_CKBOOL verify_array( a_Array_Sub array );
@@ -914,7 +965,11 @@ Chuck_Type * new_array_type( Chuck_Env * env, Chuck_Type * array_parent,
 // make type | 1.4.1.1 (nshaheed) added
 Chuck_Type * new_array_element_type( Chuck_Env * env, Chuck_Type * base_type,
                                      t_CKUINT depth, Chuck_Namespace * owner_nspc);
+
+
+//-----------------------------------------------------------------------------
 // conversion
+//-----------------------------------------------------------------------------
 const char * type_path( a_Id_List path );
 a_Id_List str2list( const std::string & path );
 a_Id_List str2list( const std::string & path, t_CKBOOL & is_array );
@@ -922,7 +977,21 @@ const char * howmuch2str( te_HowMuch how_much );
 t_CKBOOL escape_str( char * str_lit, int linepos );
 t_CKINT str2char( const char * char_lit, int linepos );
 
+
+//-----------------------------------------------------------------------------
+// more helper functions for type scan and checking | 1.5.0.0 (ge) added
+//-----------------------------------------------------------------------------
+// compare two argument lists to see if they are the same (sequence of types)
+t_CKBOOL same_arg_lists( a_Arg_List lhs, a_Arg_List rhs );
+// generate a string from an argument list (types only)
+std::string arglist2string( a_Arg_List list );
+
+
+
+
+//-----------------------------------------------------------------------------
 // REFACTOR-2017: exile! these default types now stored in env
+//-----------------------------------------------------------------------------
 //extern Chuck_Type t_void;
 //extern Chuck_Type t_int;
 //extern Chuck_Type t_float;
