@@ -39,7 +39,7 @@
 #include <math.h>
 #include <stdio.h>
 
-static t_CKUINT g_srate = 0;
+static t_CKUINT g_srateOsc = 0;
 // for member data offset
 static t_CKUINT osc_offset_data = 0;
 
@@ -51,7 +51,7 @@ static t_CKUINT osc_offset_data = 0;
 DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
 {
     // srate
-    g_srate = QUERY->srate;
+    g_srateOsc = QUERY->srate;
     // get the env
     Chuck_Env * env = QUERY->env();
 
@@ -136,6 +136,9 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
                                         doc.c_str() ) )
         return FALSE;
 
+    // add examples | 1.5.0.0 added
+    if( !type_engine_import_add_ex( env, "basic/phasor.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -148,7 +151,15 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
                                         doc.c_str() ) )
         return FALSE;
 
+    // add examples | 1.5.0.0 (ge)
+    type_engine_import_add_ex( env, "otf_05.ck" );
+    type_engine_import_add_ex( env, "otf_06.ck" );
+    type_engine_import_add_ex( env, "basic/foo.ck" );
+    type_engine_import_add_ex( env, "basic/bar.ck" );
+    type_engine_import_add_ex( env, "basic/chirp.ck" );
+    type_engine_import_add_ex( env, "basic/alarm.ck" );
     type_engine_import_add_ex( env, "basic/whirl.ck" );
+    type_engine_import_add_ex( env, "analysis/fft.ck" );
 
     // end the class import
     type_engine_import_class_end( env );
@@ -172,6 +183,10 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
     func->doc = "get width of triangle wave (ratio of rise time to fall time).";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "basic/oscillatronx.ck" ) ) goto error;
+    if( !type_engine_import_add_ex( env, "deep/shepard.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -194,6 +209,10 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
     func->doc = "whether sawtooth wave is to fall (0) or rise (1).";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "deep/thx.ck" ) ) goto error;
+    if( !type_engine_import_add_ex( env, "basic/oscillatronx.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -216,6 +235,9 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
     func->doc = "get width of duty cycle [0,1)/";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "basic/pulse.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -233,6 +255,12 @@ DLL_QUERY osc_query( Chuck_DL_Query * QUERY )
     func->doc = "set width of duty cycle (always 0.5).";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "basic/oscillatronx.ck" ) ) goto error;
+    if( !type_engine_import_add_ex( env, "filter/brf.ck" ) ) goto error;
+    if( !type_engine_import_add_ex( env, "filter/rlpf.ck" ) ) goto error;
+    if( !type_engine_import_add_ex( env, "filter/rhpf.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -273,7 +301,7 @@ struct Osc_Data
         freq = 220.0;
         sync = 0; // internal
         width = 0.5;
-        srate = g_srate;
+        srate = g_srateOsc;
         phase = 0.0;
     }
 };
@@ -911,7 +939,7 @@ static void _transition( t_CKDOUBLE a, t_CKDOUBLE alpha, t_CKDOUBLE b,
 DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
 {
     // srate
-    g_srate = QUERY->srate;
+    g_srateOsc = QUERY->srate;
     // get the env
     Chuck_Env * env = QUERY->env();
     std::string doc;
@@ -949,6 +977,9 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients; meaning is dependent on subclass";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/readme-GenX.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -973,11 +1004,13 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients.";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/Gen5-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
 
-
     //---------------------------------------------------------------------
     // gen7
     //---------------------------------------------------------------------
@@ -997,6 +1030,9 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients.";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/Gen7-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -1018,11 +1054,13 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients.";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/Gen9-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
 
-
     //---------------------------------------------------------------------
     // gen10
     //---------------------------------------------------------------------
@@ -1039,6 +1077,9 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients.";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/Gen10-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
@@ -1064,11 +1105,15 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     func->doc = "set lookup table coefficients.";
     if( !type_engine_import_mfun( env, func ) ) goto error;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/Gen17-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
+
     //---------------------------------------------------------------------
-    // Curve
+    // CurveTable
     //---------------------------------------------------------------------
     doc = "constructs a wavetable composed of segments of variable times, "
     "values, and curvatures. Coefficients are specified as a single linear "
@@ -1082,6 +1127,9 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
                                         NULL, NULL, genX_tick, NULL, doc.c_str() ) )
         return FALSE;
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/GenX-CurveTable-test.ck" ) ) goto error;
+
     func = make_new_mfun( "float[]", "coefs", curve_coeffs ); //load table
     func->add_arg( "float", "v[]" );
     func->doc = "set lookup table coefficients.";
@@ -1091,10 +1139,9 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     type_engine_import_class_end( env );
 
     //---------------------------------------------------------------------
-    // Warp
+    // WarpTable
     //---------------------------------------------------------------------
-    doc = "...";
-
+    doc = "An end-constrained mapping table, mostly useful for conditioning control signals.";
     if( !type_engine_import_ugen_begin( env, "WarpTable", "GenX", env->global(),
                                         NULL, NULL, genX_tick, NULL, doc.c_str() ) )
         return FALSE;
@@ -1111,10 +1158,12 @@ DLL_QUERY genX_query( Chuck_DL_Query * QUERY )
     if( !type_engine_import_mfun( env, func ) ) goto error;
     */
 
+    // add examples
+    if( !type_engine_import_add_ex( env, "special/GenX-WarpTable-test.ck" ) ) goto error;
+
     // end the class import
     type_engine_import_class_end( env );
 
-
     return TRUE;
 
 error:
@@ -1149,7 +1198,7 @@ struct genX_Data
     {
         //initialize data here
         sync        = 0;
-        srate       = g_srate;
+        srate       = g_srateOsc;
 
         t_CKINT i;
         for( i=0; i<genX_MAX_COEFFS; i++ ) coeffs[i] = 0.;
