@@ -33,7 +33,10 @@
 #define __ULIB_MACHINE_H__
 
 #include "chuck_dl.h"
+
+#ifndef  __DISABLE_OTF_SERVER__
 #include "chuck_otf.h"
+#endif
 
 // forward reference
 struct Chuck_VM;
@@ -42,24 +45,18 @@ struct Chuck_Compiler;
 
 // query
 DLL_QUERY machine_query( Chuck_DL_Query * QUERY );
+
+#ifndef __DISABLE_OTF_SERVER__
 typedef t_CKUINT (* proc_msg_func)( Chuck_VM *, Chuck_Compiler *,
                                     Net_Msg *, t_CKBOOL, void * );
 t_CKBOOL machine_init( Chuck_Compiler * compiler, proc_msg_func func );
 // 1.4.0.1: TODO: refactor out of ulib machine; not all platforms have OTF
+#endif // __DISABLE_OTF_SERVER__
+
+// machine exports
 t_CKUINT machine_intsize();
 
 
-// exports
-CK_DLL_SFUN( machine_crash_impl );
-CK_DLL_SFUN( machine_add_impl );
-CK_DLL_SFUN( machine_spork_impl );
-CK_DLL_SFUN( machine_remove_impl );
-CK_DLL_SFUN( machine_replace_impl );
-CK_DLL_SFUN( machine_status_impl );
-CK_DLL_SFUN( machine_intsize_impl );
-CK_DLL_SFUN( machine_shreds_impl );
-CK_DLL_SFUN( machine_realtime_impl );
-CK_DLL_SFUN( machine_silent_impl );
 
 
 #endif
