@@ -29,10 +29,8 @@
 // author: Spencer Salazar (spencer@ccrma.stanford.edu)
 // date: Summer 2012
 //-----------------------------------------------------------------------------
-#include "chuck_def.h"
 #include "util_serial.h"
 #include "chuck_errmsg.h"
-using namespace std;
 
 #if defined(__MACOSX_CORE__) && !defined(__CHIP_MODE__)
 
@@ -41,6 +39,8 @@ using namespace std;
 #include <IOKit/serial/IOSerialKeys.h>
 #include <IOKit/IOBSD.h>
 
+using namespace std;
+
 vector<string> SerialIOManager::availableSerialDevices()
 {
     vector<string> devices;
@@ -121,6 +121,7 @@ cleanup:
 #include <dirent.h>
 #include <unistd.h>
 
+using namespace std;
 
 vector<string> SerialIOManager::availableSerialDevices()
 {
@@ -177,10 +178,12 @@ cleanup:
     return devices;
 }
 
-#elif defined(__WINDOWS_DS__)
+#elif defined(__PLATFORM_WIN32__)
 
 #include <windows.h>
 
+using namespace std;
+
 vector<string> SerialIOManager::availableSerialDevices()
 {
     vector<string> devices;
@@ -244,4 +247,3 @@ vector<string> SerialIOManager::availableSerialDevices()
 
 
 #endif /* __MACOSX_CORE__ */
-
