@@ -39,7 +39,7 @@
 #include <limits.h>
 #include "rtmidi.h"
 
-#if (defined(__WINDOWS_DS__) || defined(__WINDOWS_ASIO__)) && !defined(__WINDOWS_PTHREAD__)
+#if (defined(__PLATFORM_WIN32__) && !defined(__WINDOWS_PTHREAD__))
   #include <windows.h>
   #include <sys/timeb.h>
 #else
@@ -94,21 +94,100 @@ SAMPLE * ChuckAudio::m_extern_in = NULL;
 SAMPLE * ChuckAudio::m_extern_out = NULL;
 t_CKUINT ChuckAudio::m_dac_n = 0;
 t_CKUINT ChuckAudio::m_adc_n = 0;
+std::string ChuckAudio::m_dac_name = "";
+std::string ChuckAudio::m_adc_name = "";
 RtAudio * ChuckAudio::m_rtaudio = NULL;
 f_audio_cb ChuckAudio::m_audio_cb = NULL;
 void * ChuckAudio::m_cb_user_data = NULL;
 
 
+static RtAudio::Api driverNameToApi(char const *driver)
+{
+    RtAudio::Api api = RtAudio::UNSPECIFIED;
+    if(driver)
+    {
+    #if defined(__WINDOWS_ASIO__)
+        if(!strcmp(driver, "ASIO") || !strcmp(driver, "asio") || !strcmp(driver, "Asio"))
+            api = RtAudio::WINDOWS_ASIO;
+        else
+    #endif
+    #if defined(__WINDOWS_DS__)
+        if(!strcmp(driver, "DS") || !strcmp(driver, "ds") || !strcmp(driver, "Ds"))
+            api = RtAudio::WINDOWS_DS;
+    #endif
+    #if defined(__WINDOWS_WASAPI__)
+        if(!strcmp(driver, "WASAPI") || !strcmp(driver, "wasapi") || !strcmp(driver, "Wasapi"))
+            api = RtAudio::WINDOWS_WASAPI;
+    #endif
+    #if defined(__LINUX_ALSA__)
+        if(!strcmp(driver, "ALSA") || !strcmp(driver, "alsa") || !strcmp(driver, "Alsa")) // should match apiToDriverName
+            api = RtAudio::LINUX_ALSA;
+    #endif
+    #if defined(__LINUX_PULSE__)
+        if(!strcmp(driver, "Pulse") || !strcmp(driver, "pulse") || !strcmp(driver, "PULSE"))
+            api = RtAudio:: LINUX_PULSE;
+    #endif
+    #if defined(__UNIX_JACK__)
+        if(!strcmp(driver, "Jack") || !strcmp(driver, "jack") || !strcmp(driver, "JACK"))
+            api = RtAudio:: UNIX_JACK;
+    #endif
+        // XXX: add swap between ALSA, PULSE, OSS? and JACK
+        if(api == RtAudio::UNSPECIFIED) 
+        {
+            EM_error2( 0, "unsupported audio driver: %s", driver);
+        }
+    }
+
+    if(api == RtAudio::UNSPECIFIED)
+    {
+    #if defined(__PLATFORM_WIN32__)
+        // traditional chuck default behavior:
+        // DIRECT SOUND is most general albeit high-latency
+        api = RtAudio::WINDOWS_DS; 
+    #elif defined(__PLATFORM_LINUX__)
+        api = RtAudio::LINUX_PULSE;
+    #endif
+    }
+    return api;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: apiToDriverName()
+// desc: get driver name from API
+//-----------------------------------------------------------------------------
+static char const * apiToDriverName( RtAudio::Api api )
+{
+    static char const * drivers[] = {
+        "system default",
+        "ALSA",
+        "Pulse",
+        "OSS",
+        "Jack",
+        "CoreAudio",
+        "WASAPI",
+        "ASIO",
+        "DS", // DirectSound
+        "DUMMY",
+        "invalid"
+    };
+
+    return drivers[(int)api];
+}
+
+
 
 
 //-----------------------------------------------------------------------------
 // name: print()
-// desc: ...
+// desc: print info for an audio device
 //-----------------------------------------------------------------------------
 void print( const RtAudio::DeviceInfo & info )
 {
     EM_error2b( 0, "device name = \"%s\"", info.name.c_str() );
-    if (info.probed == false)
+    if (!info.probed)
         EM_error2b( 0, "probe [failed] ..." );
     else
     {
@@ -141,32 +220,41 @@ void print( const RtAudio::DeviceInfo & info )
     }
 }
 
-
+/* this is the RtAudio error handler --- */
+static void rtAudioErrorHandler(
+    RtAudioErrorType t,
+    const std::string &errorText)
+{
+    // problem finding audio devices, most likely
+    EM_error2b( 0, "%s", errorText.c_str() );
+}
 
 
 //-----------------------------------------------------------------------------
 // name: probe()
 // desc: ...
 //-----------------------------------------------------------------------------
-void ChuckAudio::probe()
+void ChuckAudio::probe( char const * driver )
 {
+    RtAudio::Api api = driverNameToApi( driver ); // handles driver=NULL case
+    char const * dnm = apiToDriverName( api );
     // rtaduio pointer
     RtAudio * audio = NULL;
     // device info struct
     RtAudio::DeviceInfo info;
     
     // allocate RtAudio
-    try { audio = new RtAudio( ); }
-    catch( RtError err )
+    audio = new RtAudio( api, rtAudioErrorHandler );
+    if( !audio )
     {
-        // problem finding audio devices, most likely
-        EM_error2b( 0, "%s", err.getMessage().c_str() );
+        // error reported by our errorHandler
         return;
     }
 
     // get count
     int devices = audio->getDeviceCount();
-    EM_error2b( 0, "found %d device(s) ...", devices );
+    EM_error2b( 0, "[%s] driver found %d audio device(s)...", dnm, devices );
+    EM_error2( 0, "" );
 
     // reset -- what does this do
     EM_reset_msg();
@@ -174,11 +262,11 @@ void ChuckAudio::probe()
     // loop over devices
     for( int i = 0; i < devices; i++ )
     {
-        try { info = audio->getDeviceInfo(i); }
-        catch( RtError & error )
-        {
-            error.printMessage();
-            break;
+        info = audio->getDeviceInfo(i);
+        if (!info.probed) { // errorHandle reports issues above
+            EM_error2(0, "");
+            EM_reset_msg();
+            continue;
         }
         
         // print
@@ -205,22 +293,22 @@ void ChuckAudio::probe()
 // desc: get device number by name; needs_dac/adc prompts further checks on
 //       requested device having > 0 channels
 //-----------------------------------------------------------------------------
-t_CKINT ChuckAudio::device_named( const std::string & name, t_CKBOOL needs_dac,
-                                   t_CKBOOL needs_adc )
+t_CKINT ChuckAudio::device_named( char const * driver,
+                                  const std::string & name,
+                                  t_CKBOOL needs_dac,
+                                  t_CKBOOL needs_adc )
 {
     // rtaudio pointer
     RtAudio * audio = NULL;
     // device info struct
     RtAudio::DeviceInfo info;
+    RtAudio::Api api = driverNameToApi(driver); // handles driver=NULL case
+    // char const * dnm = apiToDriverName(api);
     
     // allocate RtAudio
-    try { audio = new RtAudio(); }
-    catch( RtError err )
-    {
-        // problem finding audio devices, most likely
-        EM_error2b( 0, "%s", err.getMessage().c_str() );
-        return -1;
-    }
+    audio = new RtAudio(api, rtAudioErrorHandler);
+    if(!audio)
+        return -1; // reporting in errorHandler
     
     // get count    
     int devices = audio->getDeviceCount();
@@ -230,10 +318,10 @@ t_CKINT ChuckAudio::device_named( const std::string & name, t_CKBOOL needs_dac,
     for( int i = 0; i < devices; i++ )
     {
         // get info
-        try { info = audio->getDeviceInfo(i); }
-        catch( RtError & error )
+        info = audio->getDeviceInfo(i);
+        if(!info.probed)
         {
-            error.printMessage();
+            // reportError handles problem
             break;
         }
 
@@ -252,10 +340,10 @@ t_CKINT ChuckAudio::device_named( const std::string & name, t_CKBOOL needs_dac,
         // no exact match found; try partial match
         for( int i = 0; i < devices; i++ )
         {
-            try { info = audio->getDeviceInfo(i); }
-            catch( RtError & error )
+            info = audio->getDeviceInfo(i);
+            if(!info.probed)
             {
-                error.printMessage();
+                // errorHandler reports issues
                 break;
             }
             
@@ -328,7 +416,7 @@ static unsigned int __stdcall watch_dog( void * )
     t_CKUINT priority = XThreadUtil::our_priority;
 
     // log
-    EM_log( CK_LOG_SEVERE, "starting real-time watch dog processs..." );
+    EM_log( CK_LOG_SEVERE, "starting real-time watch dog process..." );
     // push log
     EM_pushlog();
     EM_log( CK_LOG_INFO, "watchdog timeout: %f::second", g_watchdog_timeout );
@@ -437,22 +525,93 @@ t_CKBOOL ChuckAudio::watchdog_stop()
 
 
 
+//-----------------------------------------------------------------------------
+// name: supportSampleRate()
+// desc: does a device (which can potentially support multiple sample rates)
+//       support a particular sample rate? 1.5.0.0 (ge) | added
+//-----------------------------------------------------------------------------
+static t_CKBOOL supportSampleRate( RtAudio::DeviceInfo & device_info, t_CKINT sample_rate )
+{
+    // loop over supported sample rates for a device
+    for( t_CKINT i = 0; i < device_info.sampleRates.size(); i++ )
+    {
+        // look for a match
+        if( device_info.sampleRates[i] == sample_rate )
+        {
+            return TRUE;
+        }
+    }
+
+    // no match
+    return FALSE;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: findMatchingInputDevice()
+// desc: find an input device with matching chans and sample rate
+//       1.5.0.0 (ge) | added
+//-----------------------------------------------------------------------------
+static t_CKINT findMatchingInputDevice( RtAudio * rtaudio, t_CKINT numInputChans,
+                                        t_CKINT sample_rate, t_CKBOOL allowMoreChans )
+{
+    // get number of devices
+    t_CKINT num_devices = rtaudio->getDeviceCount();
+    // struct to hold info for a device
+    RtAudio::DeviceInfo device_info;
+
+    // find first device with at least the requested channel count (and sample rate match)
+    for( t_CKUINT i = 0; i < num_devices; i++ )
+    {
+        // get device info
+        device_info = rtaudio->getDeviceInfo((unsigned int)i);
+        // check sample rate
+        if( supportSampleRate( device_info, sample_rate ) )
+        {
+            // check channels; the allowMoreChans toggles exact match or lenient match
+            if( (allowMoreChans == FALSE && device_info.inputChannels == numInputChans) ||
+                (allowMoreChans == TRUE && device_info.inputChannels >= numInputChans) )
+            {
+                // log
+                EM_log( CK_LOG_INFO, "found alternate input device: %d...", i );
+                // return device number
+                return i;
+            }
+        }
+    }
+
+    // no match
+    return -1;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: initialize()
 // desc: ...
 //-----------------------------------------------------------------------------
-t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
+t_CKBOOL ChuckAudio::initialize( t_CKUINT dac_device,
+                                 t_CKUINT adc_device,
+                                 t_CKUINT num_dac_channels,
                                  t_CKUINT num_adc_channels,
                                  t_CKUINT sample_rate,
                                  t_CKUINT buffer_size,
                                  t_CKUINT num_buffers,
                                  f_audio_cb callback,
                                  void * data,
-                                 t_CKBOOL force_srate )
+                                 t_CKBOOL force_srate,
+                                 char const * driver )
 {
-    if( m_init )
-        return FALSE;
+    // check if already initialized
+    if( m_init ) return FALSE;
 
+    m_dac_n = dac_device;
+    m_adc_n = adc_device;
+    m_dac_name = "[no output device]";
+    m_adc_name = "[no input device]";
     m_num_channels_out = num_dac_channels;
     m_num_channels_in = num_adc_channels;
     m_sample_rate = sample_rate;
@@ -467,8 +626,16 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
     // remember user data to pass to callback
     m_cb_user_data = data;
 
+    // audio device probe results
+    RtAudio::DeviceInfo dac_info;
+    RtAudio::DeviceInfo adc_info;
+
     // variable to pass by reference into RtAudio
     unsigned int bufsize = (unsigned int)m_buffer_size;
+    // check output device number; 0 means "default"
+    bool useDefaultOut = ( m_dac_n == 0 );
+    // check input device number; 0 means "default"
+    bool useDefaultIn = ( m_adc_n == 0 );
 
     // log
     EM_log( CK_LOG_FINE, "initializing RtAudio..." );
@@ -476,25 +643,25 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
     EM_pushlog();
 
     // allocate RtAudio
-    try { m_rtaudio = new RtAudio( ); }
-    catch( RtError err )
+    RtAudio::Api api = driverNameToApi(driver);
+    char const *dnm = apiToDriverName(api);
+    RtAudioErrorType code = RTAUDIO_NO_ERROR;
+    m_rtaudio = new RtAudio(api, rtAudioErrorHandler);
+    if(!m_rtaudio)
     {
-        // problem finding audio devices, most likely
-        EM_error2( 0, "%s", err.getMessage().c_str() );
-        // clean up
-        goto error;
+        // error reported above
+        goto error; // to clean up
     }
         
     // convert 1-based ordinal to 0-based ordinal (added 1.3.0.0)
     // note: this is to preserve previous devices numbering after RtAudio change
     if( m_num_channels_out > 0 )
     {
-        // check output device number; 0 used to mean "default"
-        bool useDefault = ( m_dac_n == 0 );
         // default (refactor 1.3.1.2)
-        if( useDefault )
+        if( useDefaultOut )
         {
             // get the default
+            // FYI: getDefaultInputDevice returns 0 either as a valid index OR if no default/any devices
             m_dac_n = m_rtaudio->getDefaultOutputDevice();
         }
         else
@@ -506,9 +673,9 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
         RtAudio::DeviceInfo device_info = m_rtaudio->getDeviceInfo((unsigned int)m_dac_n);
         
         // ensure correct channel count if default device is requested
-        if( useDefault )
+        if( useDefaultOut )
         {
-            // check
+            // check for output (channels)
             if( device_info.outputChannels < m_num_channels_out )
             {
                 // find first device with at least the requested channel count
@@ -608,20 +775,105 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
     if( m_num_channels_in > 0 )
     {
         // if we are requesting the default input device
-        if( m_adc_n == 0 )
+        if( useDefaultIn )
         {
             // get default input device
+            // FYI getDefaultInputDevice returns 0 either as a valid index OR if no default/any devices
             m_adc_n = m_rtaudio->getDefaultInputDevice();
-            
             // ensure correct channel count if default device is requested
             RtAudio::DeviceInfo device_info = m_rtaudio->getDeviceInfo((unsigned int)m_adc_n);
-            
-            // check if input channels > 0
+
+            // remember
+            t_CKUINT inChannelsRequested = m_num_channels_in;
+
+            // check if input channels > 0 || sample rate does not match
             if( device_info.inputChannels < m_num_channels_in )
             {
                 // 1.4.2.0 (ge) | added: a specific but possible common case
                 // for systems that have a 1-channel default input audio device -- e.g., some MacOS systems
-                if( m_num_channels_in == 2 && device_info.inputChannels == 1 )
+                if( m_num_channels_in == 2 && device_info.inputChannels == 1
+                    && supportSampleRate( device_info, sample_rate ) ) // try to stay on device if possible
+                {
+                    // nothing more for now here
+                    // the code here to set the m_expand_in_mono2stereo flag has been moved below
+                }
+                else // need further matching
+                {
+                    // special case | 1.5.0.0 (ge) -- in this case of default input device,
+                    // first try to match using the current default input device channels.
+                    // This is to handle the situation in macOS and using bluetooth earbuds
+                    // or headphones and the highest supported input sample rate is lower than
+                    // the desired chuck sample rate (e.g., some input devices only support 24K);
+                    // in this case, look for devices with match sample rates AND inherit the
+                    // default input devices's actual channel count. This heuristic is so that
+                    // on most Apple laptops since macOS 11, this will end up choosing the
+                    // the onboard laptop microphone. This will hopefully serve many students
+                    // working in the above setup; of course, power users can explicitly choose
+                    // the input and output devices using command-line flags or other settings
+                    // -
+                    // special case | 1.4.0.1 (ge) -- check if the sample rate is supported
+                    // this is to catch the case where the default device has
+                    // insufficient channels (e.g., 1 on MacOS), finds secondary device
+                    // (e.g., ZoomAudioDevice on MacOS) that has enough channels
+                    // but does not support the desired sample rate
+                    // amended 1.4.2.0 -- the behavior is modified above, where chuck
+                    // will first check if the default input device has only 1 channel
+                    // while we are requesting 2 channels...
+                    // -
+
+                    // if non-zero input channels
+                    if( device_info.inputChannels > 0 )
+                    {
+                        // find a device with *exactly* the current device input channels
+                        m_adc_n = findMatchingInputDevice( m_rtaudio, device_info.inputChannels, sample_rate, FALSE );
+                    }
+                    else  // if device input channel count is 0
+                    {
+                        // find a device with *exactly* the requested channel count
+                        m_adc_n = findMatchingInputDevice( m_rtaudio, m_num_channels_in, sample_rate, FALSE);
+                    }
+                    
+                    // check if match
+                    if( m_adc_n >= 0 )
+                    {
+                        // get updated device info (since m_adc_n may be updated by this point)
+                        device_info = m_rtaudio->getDeviceInfo((unsigned int)m_adc_n);
+                        // update the request input channel request
+                        m_num_channels_in = device_info.inputChannels;
+                    }
+                    else
+                    {
+                        // find a device with *at least* the requested channel count
+                        m_adc_n = findMatchingInputDevice(m_rtaudio, m_num_channels_in, sample_rate, TRUE );
+
+                        // changed 1.3.1.2 (ge): for input, if no match found by this point, we just gonna try to open half-duplex
+                        if( m_adc_n < 0 )
+                        {
+                            // set to 0
+                            m_num_channels_in = 0;
+                            // log
+                            EM_log( CK_LOG_SEVERE, "cannot find compatible audio input device..." );
+                            EM_pushlog();
+                                EM_log(CK_LOG_SEVERE, "defaulting to half-duplex (no audio input)...");
+                            EM_poplog();
+
+                            // problem finding audio devices, most likely
+                            // EM_error2( 0, "unable to find audio input device with requested channel count (%i)...",
+                            //               m_num_channels_in);
+                            // clean up
+                            // goto error;
+                        }
+                    }
+                }
+            }
+
+            // if we ended up with a match
+            if( m_dac_n >= 0 )
+            {
+                // get info for the input device we ended up using
+                device_info = m_rtaudio->getDeviceInfo((unsigned int)m_adc_n);
+                // check channel situation look for specific arrangement; we should have a sample rate match if we get here
+                if( inChannelsRequested == 2 && device_info.inputChannels == 1 )
                 {
                     // flag this for expansion in the callback
                     m_expand_in_mono2stereo = TRUE;
@@ -629,88 +881,6 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
                     EM_log( CK_LOG_INFO, "using 1-channel default input audio device (instead of requested 2-channel)..." );
                     EM_log( CK_LOG_INFO, "simulating stereo input from mono audio device..." );
                 }
-                else
-                {
-                    // find first device with at least the requested channel count
-                    m_adc_n = -1;
-                    int num_devices = m_rtaudio->getDeviceCount();
-                    for( int i = 0; i < num_devices; i++ )
-                    {
-                        device_info = m_rtaudio->getDeviceInfo(i);
-                        if( device_info.inputChannels >= m_num_channels_in )
-                        {
-                            // added 1.4.0.1 -- check if the sample rate is supported
-                            // this is to catch the case where the default device has
-                            // insufficient channels (e.g., 1 on MacOS), finds secondary device
-                            // (e.g., ZoomAudioDevice on MacOS) that has enough channels
-                            // but does not support the desired sample rate
-                            // amended 1.4.2.0 -- the behavior is modified above, where chuck
-                            // will first check if the default input device has only 1 channel
-                            // while we are requesting 2 channels
-                            bool isMatch = false;
-                            for( long j = 0; j < device_info.sampleRates.size(); j++ )
-                            {
-                                if( device_info.sampleRates[j] == m_sample_rate )
-                                {
-                                    // flag
-                                    isMatch = true;
-                                    // break out sample rate loop
-                                    break;
-                                }
-                            }
-
-                            // match? (does this separately to break out of outer loop)
-                            if( isMatch )
-                            {
-                                // new input device
-                                m_adc_n = i;
-                                // log
-                                EM_log( CK_LOG_INFO, "found alternate input device: %d...", i );
-                                // break out of device loop
-                                break;
-                            }
-                        }
-                    }
-
-                    // added 1.4.0.1 (ge) -- special case if no other matching device found...
-                    // for systems that have default mono audio device e.g., some MacOS systems
-                    // (commented out due to 1.4.2.0 changes above to look mono default input first)
-                    //if( m_adc_n == -1 )
-                    //{
-                    //    // get default input device
-                    //    m_adc_n = m_rtaudio->getDefaultInputDevice();
-                    //    // get device info
-                    //    device_info = m_rtaudio->getDeviceInfo(m_adc_n);
-                    //
-                    //    // special case
-                    //    if( m_num_channels_in == 2 && device_info.inputChannels == 1 )
-                    //    {
-                    //        // flag this for expansion in the callback
-                    //        m_expand_in_mono2stereo = TRUE;
-                    //        // log
-                    //        EM_log( CK_LOG_INFO, "no matching stereo input device found..." );
-                    //        EM_log( CK_LOG_INFO, "simulating stereo input from mono audio device..." );
-                    //    }
-                    //    else
-                    //    {
-                    //        // okay, let's not impose any further; fall through to subseqent logic
-                    //        m_adc_n = -1;
-                    //    }
-                    //}
-                    
-
-                    // changed 1.3.1.2 (ge): for input, if nothing found, we just gonna try to open half-duplex
-                    if( m_adc_n == -1 )
-                    {
-                        // set to 0
-                        m_num_channels_in = 0;
-                        // problem finding audio devices, most likely
-                        // EM_error2( 0, "unable to find audio input device with requested channel count (%i)...",
-                        //               m_num_channels_in);
-                        // clean up
-                        // goto error;
-                    }
-                }
             }
         }
         else
@@ -721,10 +891,10 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
     }
 
     // open device
-    try {
+    { // was: try {
         // log
-        EM_log( CK_LOG_FINE, "trying %d input %d output...",
-                m_num_channels_in, m_num_channels_out );
+        EM_log( CK_LOG_FINE, "[%s] driver trying %d input %d output...",
+                dnm, m_num_channels_in, m_num_channels_out );
 
         RtAudio::StreamParameters output_parameters;
         output_parameters.deviceId = (unsigned int)m_dac_n;
@@ -742,20 +912,23 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
         stream_options.streamName = "ChucK";
         stream_options.priority = 0;
             
-        // open RtAudio
-        m_rtaudio->openStream(
+        // open audio stream
+        code = m_rtaudio->openStream(
             m_num_channels_out > 0 ? &output_parameters : NULL,
             m_num_channels_in > 0 ? &input_parameters : NULL,
             CK_RTAUDIO_FORMAT, (unsigned int)sample_rate, &bufsize,
             cb, m_cb_user_data,
-            &stream_options );
-    } catch( RtError err ) {
-        // log
-        EM_log( CK_LOG_INFO, "exception caught: '%s'...", err.getMessage().c_str() );
-        EM_error2( 0, "%s", err.getMessage().c_str() );
-        SAFE_DELETE( m_rtaudio );
-        // clean up
-        goto error;
+            &stream_options
+        );
+
+        // check error code
+        if( code > RTAUDIO_WARNING ) // was: } catch( RtAudioErrorType ) {
+        {
+            // RtAudio no longer throws, errors reported via errorHandler
+            SAFE_DELETE( m_rtaudio );
+            // clean up
+            goto error;
+        }
     }
         
     // check bufsize
@@ -765,6 +938,15 @@ t_CKBOOL ChuckAudio::initialize( t_CKUINT num_dac_channels,
         m_buffer_size = bufsize;
     }
 
+    // the dac and adc devices we ended up initializing | 1.5.0.0 (ge)
+    if( m_num_channels_out > 0 )
+        dac_info = m_rtaudio->getDeviceInfo((unsigned int)m_dac_n);
+    if( m_num_channels_in > 0 )
+        adc_info = m_rtaudio->getDeviceInfo((unsigned int)m_adc_n);
+    // copy names
+    m_dac_name = dac_info.name;
+    m_adc_name = adc_info.name;
+
     // pop indent
     EM_poplog();
 
@@ -900,8 +1082,9 @@ t_CKBOOL ChuckAudio::start( )
             m_rtaudio->startStream();
         m_start = TRUE;
     }
-    catch( RtError err )
+    catch( RtAudioErrorType )
     {
+        // RtAudio no longer throws, case shouldn't happen
         return FALSE;
     }
 
@@ -922,8 +1105,9 @@ t_CKBOOL ChuckAudio::stop( )
             m_rtaudio->stopStream();
         m_start = FALSE;
     }
-    catch( RtError err )
+    catch( RtAudioErrorType)
     {
+        // RtAudio no longer throws, case shouldn't happen
         return FALSE;
     }
     
@@ -962,6 +1146,10 @@ void ChuckAudio::shutdown()
     // cleanup
     SAFE_DELETE( m_rtaudio );
 
+    // deallocate | 1.5.0.0 (ge) added
+    SAFE_DELETE( m_buffer_in );
+    SAFE_DELETE( m_buffer_out );
+
     // unflag
     m_init = FALSE;
     m_start = FALSE;
