@@ -44,6 +44,10 @@
   #include <pthread.h>
 #endif
 
+using namespace std;
+
+
+
 
 //-----------------------------------------------------------------------------
 // function prototypes
@@ -167,9 +171,15 @@ static void version()
     string archs = "";
 
 #if defined(__PLATFORM_WIN32__)
-    platform = "microsoft windows";
-#elif defined(__WINDOWS_DS__)
-    platform = "microsoft windows + cygwin";
+    #if defined(__WINDOWS_ASIO__) && defined(__WINDOWS_DS__)
+        platform = "microsoft windows + DS + ASIO";
+    #elif defined(__WINDOWS_ASIO__)
+        platform = "microsoft windows + ASIO";
+    #elif defined(__WINDOWS_DS__)
+        platform = "microsoft windows + DS";
+    #else
+        platform = "microsoft windows + (?)";
+    #endif
 #elif defined(__LINUX_ALSA__)
     platform = "linux (alsa)";
 #elif defined(__LINUX_OSS__)
@@ -207,7 +217,7 @@ void usage()
     // (note: optional colon added 1.3.0.0)
     CK_FPRINTF_STDERR( "usage: chuck --[options|commands] [+-=^] file1 file2 file3 ...\n" );
     CK_FPRINTF_STDERR( "   [options] = halt|loop|audio|silent|dump|nodump|about|probe|\n" );
-    CK_FPRINTF_STDERR( "               channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|\n" );
+    CK_FPRINTF_STDERR( "               channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|driver:<name>|\n" );
     CK_FPRINTF_STDERR( "               srate:<N>|bufsize:<N>|bufnum:<N>|shell|empty|\n" );
     CK_FPRINTF_STDERR( "               remote:<hostname>|port:<N>|verbose:<N>|level:<N>|\n" );
     CK_FPRINTF_STDERR( "               callback|deprecate:{stop|warn|ignore}|\n" );
@@ -288,8 +298,25 @@ extern "C" void signal_pipe( int sig_num )
 //-----------------------------------------------------------------------------
 void global_cleanup()
 {
-    static XMutex s_mutex;
-    s_mutex.acquire();
+    // make sure we don't double clean
+    static bool s_already_cleaning = false;
+
+    // check | 1.5.0.0 (ge) added
+    if( s_already_cleaning )
+    {
+        // log | 1.5.0.0 (ge) added
+        EM_log( CK_LOG_INFO, "additional global cleanup not needed..." );
+        // done
+        return;
+    }
+
+    // log | 1.5.0.0 (ge) added
+    EM_log( CK_LOG_SYSTEM, "global cleanup initiating..." );
+    // set flag
+    s_already_cleaning = true;
+
+    // static XMutex s_mutex;
+    // s_mutex.acquire();
 
     // REFACTOR-2017: shut down audio system
     all_stop();
@@ -320,7 +347,9 @@ void global_cleanup()
     // close handle
 //    if( g_tid_otf ) CloseHandle( g_tid_otf );
 #endif
-    s_mutex.release();
+
+    // 1.5.0.0 (ge) | commented
+    // s_mutex.release();
 }
 
 
@@ -439,6 +468,7 @@ bool go( int argc, const char ** argv )
     std::string adc_name = ""; // added 1.3.0.0
     t_CKINT dac_chans = 2;
     t_CKINT adc_chans = 2;
+    t_CKINT adc_chans_before_rtaudio = adc_chans; // added 1.5.0.0
     t_CKBOOL dump = FALSE;
     t_CKBOOL probe = FALSE;
     t_CKBOOL set_priority = FALSE;
@@ -458,7 +488,11 @@ bool go( int argc, const char ** argv )
     t_CKBOOL update_otf_vm = TRUE;
     string   filename = "";
     vector<string> args;
-    
+    char const * audioDriver = NULL;
+    // dac and adc devices names | 1.5.0.0 (ge) added
+    string   dac_device_name = "";
+    string   adc_device_name = "";
+
     // list of search pathes (added 1.3.0.0)
     std::list<std::string> dl_search_path;
     // initial chug-in path (added 1.3.0.0)
@@ -493,7 +527,7 @@ bool go( int argc, const char ** argv )
     t_CKUINT count = 1;
     t_CKINT i;
 
-    //------------------------- COMMAND LINE ARGUMENTS -----------------------------
+    //------------------------- COMMAND-LINE ARGUMENTS -----------------------------
     
     // parse command line args
     for( i = 1; i < argc; i++ )
@@ -551,6 +585,8 @@ bool go( int argc, const char ** argv )
                 num_buffers = atoi( argv[i]+8 ) > 0 ? atoi( argv[i]+8 ) : num_buffers;
             else if( !strncmp(argv[i], "-n", 2) )
                 num_buffers = atoi( argv[i]+2 ) > 0 ? atoi( argv[i]+2 ) : num_buffers;
+            else if( !strncmp(argv[i], "--driver:", 9))
+                audioDriver = argv[i]+9;
             else if( !strncmp(argv[i], "--dac:", 6) ) // (added 1.3.0.0)
             {
                 // advance pointer to beginning of argument
@@ -778,7 +814,7 @@ bool go( int argc, const char ** argv )
     // probe
     if( probe )
     {
-        ChuckAudio::probe();
+        ChuckAudio::probe(audioDriver);
         
 #ifndef __DISABLE_MIDI__
         EM_error2b( 0, "" );
@@ -844,7 +880,7 @@ bool go( int argc, const char ** argv )
     if( dac_name.size() > 0 )
     {
         // check with RtAudio
-        t_CKINT dev = ChuckAudio::device_named( dac_name, TRUE, FALSE );
+        t_CKINT dev = ChuckAudio::device_named( audioDriver, dac_name, TRUE, FALSE );
         if( dev >= 0 )
         {
             dac = dev;
@@ -861,7 +897,7 @@ bool go( int argc, const char ** argv )
     if( adc_name.size() > 0 )
     {
         // check with RtAudio
-        t_CKINT dev = ChuckAudio::device_named( adc_name, FALSE, TRUE );
+        t_CKINT dev = ChuckAudio::device_named( audioDriver, adc_name, FALSE, TRUE );
         if( dev >= 0 )
         {
             adc = dev;
@@ -873,14 +909,72 @@ bool go( int argc, const char ** argv )
             exit( 1 );
         }
     }
-    
-    //------------------------- VIRTUAL MACHINE SETUP -----------------------------
-    // instantiate ChucK
+
+
+    //-------------------- VIRTUAL MACHINE SETUP (PART 1) -------------------------
+    // instantiate a ChucK!
     the_chuck = new ChucK();
+    // set log level so things can start logging
+    the_chuck->setLogLevel( log_level );
+    // log
+    EM_log( CK_LOG_SYSTEM, "booting up ChucK instance..." );
+    // push
+    EM_pushlog();
+
+    //--------------------------- AUDIO I/O SETUP ---------------------------------
+    // log
+    EM_log( CK_LOG_SYSTEM, "initializing audio subsystem..." );
+    // push
+    EM_pushlog();
+    // log
+    // EM_log( CK_LOG_SYSTEM, "setting up for '%s' audio I/O...", g_enable_realtime_audio ? "real-time" : "silent mode" );
     
-    // set params
+    // remember this, since adc_chans could be altered when initialization real-time audio
+    adc_chans_before_rtaudio = adc_chans;
+
+    // initialize audio system
+    if( g_enable_realtime_audio )
+    {
+        // initialize real-time audio
+        t_CKBOOL retval = ChuckAudio::initialize( dac, adc, dac_chans, adc_chans,
+            srate, buffer_size, num_buffers, cb, (void *)the_chuck, force_srate, audioDriver );
+        // check return code
+        if( !retval )
+        {
+            EM_log( CK_LOG_SYSTEM, "cannot initialize audio device..." );
+            EM_log( CK_LOG_SYSTEM, "| (use --probe see list of available audio devices)" );
+            EM_log( CK_LOG_SYSTEM, "| (use --dac/--adc to explicitly select output/input devices)" );
+            EM_log( CK_LOG_SYSTEM, "| (use --silent/-s for non-realtime)" );
+            EM_log( CK_LOG_SYSTEM, "| (use --help for more details)" );
+            // pop
+            EM_poplog();
+            // done
+            exit( 1 );
+        }
+
+        // these could have been updated during initialization
+        adc = ChuckAudio::m_adc_n+1;
+        dac = ChuckAudio::m_dac_n+1;
+        adc_chans = ChuckAudio::m_num_channels_in;
+        dac_chans = ChuckAudio::m_num_channels_out;
+        adc_device_name = ChuckAudio::m_adc_name;
+        dac_device_name = ChuckAudio::m_dac_name;
+        srate = ChuckAudio::m_sample_rate;
+        buffer_size = ChuckAudio::buffer_size();
+        num_buffers = ChuckAudio::num_buffers();
+    }
+    
+    // pop
+    EM_poplog();
+
+
+    //-------------------- VIRTUAL MACHINE SETUP (PART 2) -------------------------
+    // set params (some, like sample rate, could have been updated during
+    // real-time audio initialization, so we are setting actual params here)
+    // set the actual sample rate; needed to map to in-language durations
     the_chuck->setParam( CHUCK_PARAM_SAMPLE_RATE, srate );
-    the_chuck->setParam( CHUCK_PARAM_INPUT_CHANNELS, adc_chans );
+    // use the request number for adc # channels
+    the_chuck->setParam( CHUCK_PARAM_INPUT_CHANNELS, adc_chans_before_rtaudio );
     the_chuck->setParam( CHUCK_PARAM_OUTPUT_CHANNELS, dac_chans );
     the_chuck->setParam( CHUCK_PARAM_VM_ADAPTIVE, adaptive_size );
     the_chuck->setParam( CHUCK_PARAM_VM_HALT, (t_CKINT)(vm_halt) );
@@ -894,43 +988,20 @@ bool go( int argc, const char ** argv )
     the_chuck->setParam( CHUCK_PARAM_USER_CHUGIN_DIRECTORIES, dl_search_path );
     // set hint, so internally can advise things like async data writes etc.
     the_chuck->setParam( CHUCK_PARAM_HINT_IS_REALTIME_AUDIO, g_enable_realtime_audio );
-    the_chuck->setLogLevel( log_level );
-    
+
     // initialize
     if( !the_chuck->init() )
     {
         CK_FPRINTF_STDERR( "[chuck]: failed to initialize...\n" );
         exit( 1 );
     }
-    
-    //--------------------------- AUDIO I/O SETUP ---------------------------------
+
+    //------------------ PRINT VERIFIED AUDIO SETTING ---------------------------
+
     // log
-    EM_log( CK_LOG_SYSTEM, "initializing audio I/O..." );
+    EM_log( CK_LOG_SYSTEM, "audio subsystem enabled..." );
     // push
     EM_pushlog();
-    // log
-    EM_log( CK_LOG_SYSTEM, "probing '%s' audio subsystem...", g_enable_realtime_audio ? "real-time" : "fake-time" );
-    
-    // initialize audio system
-    if( g_enable_realtime_audio )
-    {
-        // TODO: refactor initialize() to take in the dac and adc nums
-        ChuckAudio::m_adc_n = adc;
-        ChuckAudio::m_dac_n = dac;
-        t_CKBOOL retval = ChuckAudio::initialize( dac_chans, adc_chans,
-            srate, buffer_size, num_buffers, cb, (void *)the_chuck, force_srate );
-        // check
-        if( !retval )
-        {
-            EM_log( CK_LOG_SYSTEM,
-                   "cannot initialize audio device (use --silent/-s for non-realtime)" );
-            // pop
-            EM_poplog();
-            // done
-            exit( 1 );
-        }
-    }
-    
     // log
     EM_log( CK_LOG_SYSTEM, "real-time audio: %s", g_enable_realtime_audio ? "YES" : "NO" );
     EM_log( CK_LOG_SYSTEM, "mode: %s", block ? "BLOCKING" : "CALLBACK" );
@@ -939,47 +1010,42 @@ bool go( int argc, const char ** argv )
     if( g_enable_realtime_audio )
     {
         EM_log( CK_LOG_SYSTEM, "num buffers: %ld", num_buffers );
-        EM_log( CK_LOG_SYSTEM, "adc: %ld dac: %d", adc, dac );
         EM_log( CK_LOG_SYSTEM, "adaptive block processing: %ld", adaptive_size > 1 ? adaptive_size : 0 );
+        // EM_log( CK_LOG_SYSTEM, "adc: %ld dac: %d", adc, dac );
+        EM_log( CK_LOG_SYSTEM, "adc:[%d] \"%s\"", adc, adc_device_name.c_str() );
+        EM_log( CK_LOG_SYSTEM, "dac:[%d] \"%s\"", dac, dac_device_name.c_str() );
     }
     EM_log( CK_LOG_SYSTEM, "channels in: %ld out: %ld", adc_chans, dac_chans );
-    
     // pop
     EM_poplog();
 
+    //------------------------- HID RELATED SETUP --------------------------------
 #ifndef __ALTER_HID__
     // pre-load hid
     if( load_hid ) HidInManager::init();
 #endif // __ALTER_HID__
+
     
+    //------------------------- CLI RELATED SETUP --------------------------------
     // catch SIGINT
     signal( SIGINT, signal_int );
 #ifndef __PLATFORM_WIN32__
     // catch SIGPIPE
     signal( SIGPIPE, signal_pipe );
 #endif
-    
-    // shell initialization
-    if( g_enable_shell )
-    {
-        // instantiate
-        g_shell = new Chuck_Shell;
-        // initialize
-        if( !init_shell( g_shell, new Chuck_Console(), the_chuck->vm() ) )
-            exit( 1 );
-    }
 
-    // reset count
-    count = 1;
-    
+    // version
+    EM_log( CK_LOG_SYSTEM, "ChucK version: %s", the_chuck->version() );
+
+    //------------------------- SOURCE COMPILATION --------------------------------
     // log
     EM_log( CK_LOG_SEVERE, "starting compilation..." );
     // push indent
     EM_pushlog();
-    
-    
-    //------------------------- SOURCE COMPILATION --------------------------------
-    
+
+    // reset count
+    count = 1;
+
     // loop through and process each file
     for( i = 1; i < argc; i++ )
     {
@@ -1023,21 +1089,31 @@ bool go( int argc, const char ** argv )
     
     
     //----------------------- SHELL SETUP -----------------------------
-    
-    // start shell on separate thread | REFACTOR-2017: per-VM?!?
+
+    // shell initialization
     if( g_enable_shell )
     {
+        // instantiate
+        g_shell = new Chuck_Shell;
+        // initialize
+        if( !init_shell( g_shell, new Chuck_Console(), the_chuck->vm() ) )
+            exit( 1 );
+
         // flag
         g_shutdown_shell = FALSE;
+
+        // start shell on separate thread | REFACTOR-2017: per-VM?!?
 #if !defined(__PLATFORM_WIN32__) || defined(__WINDOWS_PTHREAD__)
         pthread_create( &g_tid_shell, NULL, shell_cb, g_shell );
 #else
         g_tid_shell = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)shell_cb, g_shell, 0, 0 );
 #endif
     }
+
+    // pop log
+    EM_poplog();
     
     //-------------------------- MAIN CHUCK LOOP!!! -----------------------------
-    
     // log
     EM_log( CK_LOG_SYSTEM, "running main loop..." );
 
@@ -1046,10 +1122,6 @@ bool go( int argc, const char ** argv )
     
     // push indent
     EM_pushlog();
-    // log
-    EM_log( CK_LOG_SYSTEM, "starting virtual machine..." );
-    // pop indent
-    EM_poplog();
     
     // silent mode buffers
     SAMPLE * input = new SAMPLE[buffer_size*adc_chans];
@@ -1061,10 +1133,18 @@ bool go( int argc, const char ** argv )
     // if real-time audio mode
     if( g_enable_realtime_audio )
     {
+        // log
+        EM_log( CK_LOG_SYSTEM, "starting real-time audio..." );
         // start real-time audio I/O
         ChuckAudio::start();
     }
     
+    // log
+    EM_log( CK_LOG_SYSTEM, "starting virtual machine..." );
+
+    // pop indent
+    EM_poplog();
+
     // wait
     while( the_chuck->vm_running() )
     {
